// NRF24L01+ demo in C for the NXP LPC1114FN28
// Author: Frank Duignan.  
// Updates posted on http://ioprog.com and http://eleceng.dit.ie/frank

/* This program interfaces to the nrf24l01 radio module
 * It uses the UART to send data back to the host using
 * simple print statements.  UART transmission and reception 
 * is interrupt driven.  
 * 
 * Operation:
 * 
 * Wiring.
 * A USB to serial converter is wired to the TX (pin 16 )and RX (pin 15) pins of UART0
 * SPI0 is used.
 * SSEL0 is on PIO0_2 (Pin 25)
 * SCK0 is on PIO_6 (Pin 6)
 * MISO0 is on PIO0_8 (Pin 1)
 * MOSI0 is on PIO0_9 (Pin 2)
 */
#include "lpc111x.h"
#include "serial.h"
#include "nrf24l01.h"
void delay(unsigned dly)
{ // approximate 1us delay routine at 48MHz
	int inner;
	while(dly--)
	{
		inner=2;
		while(inner--)
		{			
			asm(" NOP ");
		};
		
	};
}
void printRegisters()
{
	int rx;	
	int regnum;
	for (regnum = 0; regnum < 0x20; regnum++)
	{
		printInteger(regnum);
		printString(" : ");
		rx = NRFReadRegister(regnum);
		printInteger(rx);
		printString("\r\n");
		delay(100000); // must delay here to allow serial comms to catch up
	}	    
}
int main() 
{	
	
	uint8_t payload[10];
	
	int index;
	initUART();
    NRFinit();   	
	enable_interrupts();	
	printRegisters();
	NRFWriteData(sizeof(payload),payload);		
	delay(1000000);	
	NRFWriteCE(1);
	printRegisters();
	payload[0]=0;
	while(1) 
	{							                      
		NRFFlushTX();
		NRFFlushRX();	
		payload[0]++;
		NRFWriteData(sizeof(payload),payload);				
		delay(100000);
		NRFEnableRXMode();
		// check status register to see if there is data
		if ((NRFReadRegister(7) & 0x40))
		{
			printString("Got Data ");
			NRFReadData(sizeof(payload),payload);				
			printInteger(payload[1]);
			printString("\r\n");
		}
	}    
}



