// spi.h for LPC1114
// Author: Frank Duignan.  
// Updates posted on http://ioprog.com and http://eleceng.dit.ie/frank

#include "lpc111x.h"
#include <stdint.h>
#include "spi.h"
void initSPI(int bits, int mode)
{
	int drain_count,drain;
	// Will not be using the built-in SSEL0 line as there seems to be 
	// an incompatibilty between its use and the nrf24L01.  I suspect
	// it has to do with FIFO management in the LPC1114.  In any
	// event, pins 1 to 4 will be used for the SPI interface - nice & neat
    // will default to a speed of 1MHz
    SYSAHBCLKCTRL |= BIT16; // turn on IOCON
    IOCON_PIO0_2 = 0x11;// select SSEL0 function and enable pullup
    IOCON_SCK0_LOC = 0; // route SCK0 towards PIO0_10
    IOCON_SWCLK_PIO0_10 = 0x12;// select SCK0 function and enable pullup
    IOCON_PIO0_8 = 0x11; // select MISO0 function and enable pullup
    IOCON_PIO0_9 = 0x11; // select MOSI0 function and enable pullup
    SYSAHBCLKCTRL |= BIT11; // turn on SSP0 
    PRESETCTRL &= ~BIT0; // ensure SPI0 is in reset
    PRESETCTRL |= BIT0; // take SPI0 out of reset
    // Next need to set up the SPI frame format and speed
    // PCLK = 48MHz.  
    SSP0CLKDIV = 1; // divide down PCLK by 1 
    SSP0CPSR = 48; // Set CPS divisor to 48 to give clock rate of 1MHz
    SSP0CR0 = (bits-1) + (mode << 6) ;
    SSP0CR1 =  BIT1; // select master mode and enable SPI  
    
    SYSAHBCLKCTRL |= BIT6; // Turn on GPIO
    IOCON_R_PIO0_11 = BIT7 + BIT0; // Make R_PIO0_11 behave as a standard output
    GPIO0DIR |= BIT11 + BIT5; // Make GPIO0_11 and GPIO0_5 outputs
    GPIO0DATA |= BIT11; 	// Make CSN output initially high
    GPIO0DATA &= ~BIT5;		// Make CE output initially low       
    for (drain_count = 0; drain_count < 32; drain_count++)
		drain = transferSPI(0x00);
}

uint8_t transferSPI(uint8_t data)
{
    unsigned Timeout = 1000000;
    int ReturnValue;
	
    while (((SSP0SR & BIT4)!=0)&&(Timeout--));
    SSP0DR = data;
    Timeout = 1000000;
    while (((SSP0SR & BIT4)!=0)&&(Timeout--));        
	ReturnValue = SSP0DR;
	
    return ReturnValue;
}

